#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@package cgsn_parsers.parsers.parse_spkir
@file cgsn_parsers/parsers/parse_spkir.py
@author Christopher Wingard
@brief Parses SPKIR data logged by the custom built WHOI data loggers.
"""
import os
import re

from struct import Struct

# Import common utilities and base classes
from cgsn_parsers.parsers.common import ParserCommon
from cgsn_parsers.parsers.common import dcl_to_epoch, inputs

# Regex pattern for a line with a DCL time stamp and the OCR-507 data sample
DCL_TIMESTAMP = b'(\d{4}/\d{2}/\d{2}\s\d{2}:\d{2}:\d{2}.\d{3})'
FLOAT = b'([+-]?\d+.\d+[Ee]?[+-]?\d*)'  # includes scientific notation
NEWLINE = b'(?:\r\n|\n)?'
PATTERN = (
    DCL_TIMESTAMP + b'\s+' +     # DCL Time-Stamp
    b'SATDI7' + b'([\d]{4})' +   # Serial number
    FLOAT +                      # Timer (seconds)
    b'([\x00-\xFF]{38})' +       # binary data packet
    NEWLINE
)
REGEX = re.compile(PATTERN, re.DOTALL)

# Set the format for the binary packet for later unpacking
SPKIR = Struct('>h7I3H2B')

_parameter_names_spkir = [
        'date_time_string',
        'serial_number',
        'timer',
        'sample_delay',
        'raw_channels',
        'input_voltage',
        'analog_rail_voltage',
        'frame_counter',
        'internal_temperature'
    ]


class Parser(ParserCommon):
    """
    A Parser subclass that calls the Parser base class, adds the SPKIR specific
    methods to parse the data, and extracts the SPKIR data records from the DCL
    daily log files.
    """
    def __init__(self, infile):
        self.initialize(infile, _parameter_names_spkir)

    def parse_data(self):
        """
        Iterate through the record lines (defined via the regex expression
        above) in the data object, and parse the data into a pre-defined
        dictionary object created using the Bunch class.
        """
        # find all the spkir data packets
        for match in REGEX.findall(self.raw):
            self._build_parsed_values(match)

    def _build_parsed_values(self, match):
        """
        Extract the data from the relevant regex groups and assign to elements
        of the data dictionary.
        """
        # Use the date_time_string to calculate an epoch timestamp (seconds since
        # 1970-01-01)
        epts = dcl_to_epoch(match[0].decode('utf-8'))
        self.data.time.append(epts)

        self.data.date_time_string.append(match[0].decode('utf-8'))
        self.data.serial_number.append(int(match[1]))
        self.data.timer.append(float(match[2]))

        # Unpack the binary data packet
        (delay, ch1, ch2, ch3, ch4, ch5, ch6, ch7,
         Vin, Va, temp, count, check) = SPKIR.unpack(match[3])

        # Assign the remaining SPKIR data to the named parameters
        self.data.sample_delay.append(delay)
        self.data.raw_channels.append([ch1, ch2, ch3, ch4, ch5, ch6, ch7])
        self.data.input_voltage.append(Vin)
        self.data.analog_rail_voltage.append(Va)
        self.data.frame_counter.append(count)
        self.data.internal_temperature.append(temp)

def main(argv=None):
    # load the input arguments
    args = inputs(argv)
    infile = os.path.abspath(args.infile)
    outfile = os.path.abspath(args.outfile)

    # initialize the Parser object for SPKIR
    spkir = Parser(infile)

    # load the data into a buffered object and parse the data into a dictionary
    spkir.load_binary()
    spkir.parse_data()

    # write the resulting Bunch object via the toJSON method to a JSON
    # formatted data file (note, no pretty-printing keeping things compact)
    with open(outfile, 'w') as f:
        f.write(spkir.data.toJSON())

if __name__ == '__main__':
    main()
